

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

#include "gdal.h"
#include "cpl_string.h"


//===========================================================================
void UsageMessage()
{
  printf("\n");
  printf("gdalTestGCPWrite <srcTiffFileName> <dstTiffFileName>\n");
  printf("\n");
  exit(0);
}


//===========================================================================
int main(int argc, char* argv[])
{
  GDAL_GCP* GCPs = NULL;
  GDALDatasetH srcImage;
  GDALDatasetH dstImage;
  int xSize;
  int ySize;
  int x;
  int y;
  int index;
  int numGCPs;
  double adfGeoTransform[6] = {0.0, 1.0, 0.0, 0.0, 0.0, 1.0};

  GDALAllRegister();

  if(argc < 3)
    UsageMessage();

  // attempts to open the image
  srcImage = GDALOpen(CPLStrdup(argv[1]), GA_ReadOnly);
  if(srcImage == NULL)
  {
    printf("ERROR:  Failed to open (%s).\n", argv[1]);
    return 1;
  }

  // attempts to copy the srcImage
  dstImage = GDALCreateCopy(GDALGetDriverByName("GTiff"), CPLStrdup(argv[2]), srcImage, 1, NULL, NULL, NULL);
  if(dstImage == NULL)
  {
    printf("ERROR:  Failed to create copy (%s).\n", argv[2]);
    return 1;
  }

  // gets the size of dstImage
  xSize = GDALGetRasterXSize(dstImage);
  ySize = GDALGetRasterYSize(dstImage);
  printf("x size = %d\n", xSize);
  printf("y size = %d\n", ySize);
  numGCPs = (xSize * ySize);
  printf("number of GCPs = %d\n", numGCPs);

  // creates the array of gcps
  GCPs = (GDAL_GCP*)CPLMalloc(sizeof(GDAL_GCP) * numGCPs);
  GDALInitGCPs(numGCPs, GCPs);
  for(y = 0; y < ySize; ++y)
  {
    for(x = 0; x < xSize; ++x)
    {
      index = (y * xSize) + x;
      GCPs[index].dfGCPPixel = x;
      GCPs[index].dfGCPLine = y;
      GCPs[index].dfGCPX = 1.0;
      GCPs[index].dfGCPY = 1.0;
      GCPs[index].dfGCPZ = 0.0;
    }
  }

  // sets the dstImage geotransform
  GDALSetGeoTransform(dstImage, adfGeoTransform);

  // sets the dstImage GCPs
  if(GDALSetGCPs(dstImage, numGCPs, GCPs, CPLStrdup("+proj=longlat +ellps=WGS84 +datum=WGS84")) != CE_None)
  {
    printf("SetGCPs failed - %d, %s\n", CPLGetLastErrorNo(), CPLGetLastErrorMsg());
  }
  else
  {
    printf("GCP count (%d)\n", GDALGetGCPCount(dstImage));
  }

  // closes the images
  GDALClose(srcImage);
  GDALClose(dstImage);

  return 0;
}

